<?php

namespace QUI\ERP\CRM;

use QUI;

/**
 * Class Handler
 *
 * @package QUI\ERP\CRM
 */
class Task
{
    /**
     * @var integer
     */
    protected $id;

    /**
     * @var array
     */
    protected $data;

    /**
     * @var QUI\ERP\Comments
     */
    protected $Comments;

    /**
     * Task constructor.
     *
     * @param $taskId
     *
     * @throws TaskException
     */
    public function __construct($taskId)
    {
        $this->id = (int)$taskId;

        try {
            $this->data = TaskHandler::getTaskData($this->id);
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::addAlert($Exception->getMessage());

            throw new TaskException(['quiqqer/erp', 'exception.task.error']);
        }

        // comments
        $this->Comments = new QUI\ERP\Comments();

        if (isset($this->data['comments'])) {
            $this->Comments = QUI\ERP\Comments::unserialize($this->data['comments']);
        }
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return [
            'id'       => $this->getId(),
            'title'    => $this->getTitle(),
            'text'     => $this->getDescription(),
            'type'     => $this->getType(),
            'to'       => $this->getTo(),
            'from'     => $this->getFrom(),
            'userId'   => $this->getUserId(),
            'editorId' => $this->getEditorId(),
            'status'   => $this->getStatus(),
            'comments' => $this->getComments()->toArray()
        ];
    }

    /**
     * saves the current task
     */
    public function save()
    {
        try {
            QUI::getDataBase()->update(
                TaskHandler::table(),
                [
                    'title'    => $this->getTitle(),
                    'text'     => $this->getText(),
                    'taskType' => $this->getType(),
                    'to'       => $this->getTo(),
                    'from'     => $this->getFrom(),
                    'userId'   => $this->getUserId(),
                    'editorId' => $this->getEditorId(),
                    'status'   => $this->getStatus(),
                    'comments' => $this->getComments()->toJSON()
                ],
                ['id' => $this->getId()]
            );
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeException($Exception);;
        }
    }

    //region getter

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Return the title of the task
     *
     * @return string
     */
    public function getTitle()
    {
        return $this->data['title'];
    }

    /**
     * Return the text / description of the task
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->data['text'];
    }

    /**
     * Alias for getDescription()
     *
     * @return string
     */
    public function getText()
    {
        return $this->getDescription();
    }

    /**
     * @return mixed
     */
    public function getType()
    {
        return $this->data['taskType'];
    }

    /**
     * @return mixed
     */
    public function getTo()
    {
        return $this->data['to'];
    }

    /**
     * @return mixed
     */
    public function getFrom()
    {
        return $this->data['from'];
    }

    /**
     * @return mixed
     */
    public function getStatus()
    {
        return $this->data['status'];
    }

    //endregion

    //region setter

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->data['title'] = $title;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->data['text'] = $description;
    }

    /**
     * @param string $description
     */
    public function setText($description)
    {
        $this->setDescription($description);
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->data['taskType'] = $type;
    }

    /**
     * @param string|integer $to
     */
    public function setTo($to)
    {
        // unix timestamp
        if (\is_int($to)) {
            $to = \date('Y-m-d H:i:s', $to);
        }

        if (!QUI\Utils\Security\Orthos::checkMySqlDatetimeSyntax($to)) {
            return;
        }

        $this->data['to'] = $to;
    }

    /**
     * @param string|integer $from
     */
    public function setFrom($from)
    {
        // unix timestamp
        if (\is_int($from)) {
            $from = \date('Y-m-d H:i:s', $from);
        }

        if (!QUI\Utils\Security\Orthos::checkMySqlDatetimeSyntax($from)) {
            return;
        }

        $this->data['from'] = $from;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        $this->data['status'] = $status;
    }

    //endregion

    //region user

    /**
     * @return int|null
     */
    public function getUserId()
    {
        if (empty($this->data['userId'])) {
            return null;
        }

        return (int)$this->data['userId'];
    }

    /**
     * @return false|QUI\Users\Nobody|QUI\Users\SystemUser|QUI\Users\User|null
     */
    public function getUser()
    {
        if ($this->getUserId() === null) {
            return null;
        }

        try {
            return QUI::getUsers()->get($this->getUserId());
        } catch (\Exception $Exception) {
            return null;
        }
    }

    /**
     * @param $userId
     */
    public function setUserId($userId)
    {
        try {
            $User = QUI::getUsers()->get($userId);
        } catch (\Exception $Exception) {
            return;
        }

        $this->data['userId'] = $User->getId();
    }

    //endregion

    //region editor

    /**
     * @return int|null
     */
    public function getEditorId()
    {
        if (empty($this->data['editorId'])) {
            return null;
        }

        return (int)$this->data['editorId'];
    }

    /**
     * @return false|QUI\Users\Nobody|QUI\Users\SystemUser|QUI\Users\User|null
     */
    public function getEditor()
    {
        if ($this->getEditorId() === null) {
            return null;
        }

        try {
            return QUI::getUsers()->get($this->getEditorId());
        } catch (\Exception $Exception) {
            return null;
        }
    }

    /**
     * @param $editorId
     */
    public function setEditorId($editorId)
    {
        try {
            $User = QUI::getUsers()->get($editorId);
        } catch (\Exception $Exception) {
            return;
        }

        $this->data['editorId'] = $User->getId();
    }

    //endregion

    //region comments

    /**
     * @return QUI\ERP\Comments
     */
    public function getComments()
    {
        return $this->Comments;
    }

    /**
     * Add a comment to the task
     *
     * @param $text
     */
    public function addComment($text)
    {
        $this->Comments->addComment(
            $text,
            false,
            'quiqqer/crm',
            'fa fa-edit'
        );
    }

    //endregion
}
